'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _leftPad = require('left-pad');

var _leftPad2 = _interopRequireDefault(_leftPad);

var _co = require('co');

var _co2 = _interopRequireDefault(_co);

var _coPrompt = require('co-prompt');

var _coPrompt2 = _interopRequireDefault(_coPrompt);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _jsonStableStringify = require('json-stable-stringify');

var _jsonStableStringify2 = _interopRequireDefault(_jsonStableStringify);

var _config = require('../config');

var _upload = require('../upload');

var _print = require('./print');

var _print2 = _interopRequireDefault(_print);

var _github = require('../github/github');

var _gist = require('../github/gist');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function () {
  var gist = (0, _config.getConfig)('repository.gist');

  var getGistAndUpload = gist && gist.length ? (0, _gist.get)(gist).then(function (r) {
    return r && r.data && r.data.files ? executeUpload() : Promise.reject({ code: 404 });
  }) : Promise.reject({ code: 404 });

  getGistAndUpload.catch(function (e) {
    // Gist id is not set or invalid gist id
    if (e && e.code === 404) {
      (0, _co2.default)(regeneratorRuntime.mark(function _callee() {
        var gistId, createOrGet, response, publicity;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _print2.default.error('Invalid gist', 'Gist id is not set or invalid.');
                _context.next = 3;
                return (0, _coPrompt2.default)(_chalk2.default.green('Gist id(Empty to create a new gist): '));

              case 3:
                gistId = _context.sent;
                createOrGet = void 0;

                if (!(gistId && gistId.length)) {
                  _context.next = 9;
                  break;
                }

                createOrGet = (0, _gist.get)(gistId).then(function () {
                  return (0, _config.setConfig)('repository.gist', gistId);
                });
                _context.next = 15;
                break;

              case 9:
                _context.next = 11;
                return (0, _coPrompt2.default)(_chalk2.default.green('Do you want to make it public?(y/n)'));

              case 11:
                response = _context.sent;
                publicity = ['y', 'yes'].indexOf(response.toLowerCase()) !== -1;


                _print2.default.info('Gist will be created in ' + _chalk2.default.bold.white(publicity ? 'pubic' : 'private') + '.');

                createOrGet = (0, _gist.create)({
                  files: (0, _upload.getUploadingFiles)(),
                  public: publicity
                });

              case 15:
                createOrGet.then(executeUpload).catch(function (_ref) {
                  var message = _ref.message;
                  return _print2.default.error('GIST ERROR', message);
                }).then(function () {
                  return process.stdin.pause();
                });

              case 16:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
    }
  });
};

var executeUpload = function executeUpload() {
  return (0, _upload.upload)().catch(function (err) {
    return new Promise(function (resolve, reject) {
      if (err && err.code === 401) {
        (0, _co2.default)(regeneratorRuntime.mark(function _callee2() {
          var githubToken;
          return regeneratorRuntime.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  _print2.default.error('Invalid token', 'Github Token is not valid.');
                  _context2.next = 3;
                  return (0, _coPrompt2.default)(_chalk2.default.green('Github Token: '));

                case 3:
                  githubToken = _context2.sent;

                  (0, _config.setConfig)('repository.githubToken', githubToken);
                  resolve((0, _upload.upload)());

                case 6:
                case 'end':
                  return _context2.stop();
              }
            }
          }, _callee2, this);
        }));
      } else {
        reject(err);
      }
    });
  }).then(function (_ref2) {
    var uploaded = _ref2.uploaded,
        deleted = _ref2.deleted;

    _print2.default.info('Success');

    if (uploaded.length) {
      _print2.default.info(_chalk2.default.bold.blue('Uploaded files'));
      _print2.default.info(uploaded.map(function (f) {
        return '  - ' + f;
      }).join('\n'));
    }

    if (deleted.length) {
      _print2.default.info(_chalk2.default.bold.magenta('Deleted files'));
      _print2.default.info(deleted.map(function (f) {
        return '  - ' + f;
      }).join('\n'));
    }
  });
};