'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.removeConfig = exports.setConfig = exports.getConfigs = exports.getConfig = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require('fs');

var fs = _interopRequireWildcard(_fs);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _fsExtra = require('fs-extra');

var _mkdirp = require('mkdirp');

var _rc = require('rc');

var _rc2 = _interopRequireDefault(_rc);

var _jsonStableStringify = require('json-stable-stringify');

var _jsonStableStringify2 = _interopRequireDefault(_jsonStableStringify);

var _lodash = require('lodash');

var _keys = require('./keys');

var _paths = require('../paths');

var _defaultConfig = require('./defaultConfig');

var _defaultConfig2 = _interopRequireDefault(_defaultConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var getConfig = exports.getConfig = function getConfig(key) {
  var config = getConfigs(key);
  return key ? config[key] : config;
};

var getConfigs = exports.getConfigs = function getConfigs() {
  for (var _len = arguments.length, keys = Array(_len), _key = 0; _key < _len; _key++) {
    keys[_key] = arguments[_key];
  }

  var rcFile = _os2.default.type().toLowerCase().indexOf('windows') !== -1 ? fs.existsSync(_paths.configFile) ? (0, _fsExtra.readJsonSync)(_paths.configFile) : {} : (0, _rc2.default)(_paths.appName);
  var config = (0, _lodash.omit)(_extends({}, _defaultConfig2.default, rcFile), ['_', 'config', 'configs']);

  keys = keys.filter(function (k) {
    return !!k;
  });

  return keys.length ? (0, _lodash.pick)(config, keys) : config;
};

var setConfig = exports.setConfig = function setConfig(key, value) {
  (0, _mkdirp.sync)(_paths.appDir);

  var existingConfig = fs.existsSync(_paths.configFile) ? JSON.parse(fs.readFileSync(_paths.configFile)) : {};

  // Convert
  value = function (v) {
    if (!isNaN(+v)) return +v;
    switch (v) {
      case 'true':
        return true;
      case 'false':
        return false;
    }
    return v;
  }(value);

  if (!(0, _keys.validate)(key, value)) throw new Error('Invalid key value pair { \'' + key + '\' : ' + value + ' }', 'Validation Error');

  var newConfig = _extends({}, existingConfig, _defineProperty({}, key, value));

  fs.writeFileSync(_paths.configFile, (0, _jsonStableStringify2.default)(newConfig, { space: '  ' }));

  return newConfig;
};

var removeConfig = exports.removeConfig = function removeConfig(key) {
  (0, _mkdirp.sync)(_paths.appDir);

  var existingConfig = fs.existsSync(_paths.configFile) ? JSON.parse(fs.readFileSync(_paths.configFile)) : {};

  var newConfig = (0, _lodash.omit)(existingConfig, [key]);

  fs.writeFileSync(_paths.configFile, (0, _jsonStableStringify2.default)(newConfig, { space: '  ' }));

  return newConfig;
};