'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getUploadingFiles = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _jsonStableStringify = require('json-stable-stringify');

var _jsonStableStringify2 = _interopRequireDefault(_jsonStableStringify);

var _lodash = require('lodash');

var _paths = require('../paths');

var _config = require('../config');

var _gist = require('../github/gist');

var gist = _interopRequireWildcard(_gist);

var _version = require('../version');

var _version2 = _interopRequireDefault(_version);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var getUploadingFiles = exports.getUploadingFiles = function getUploadingFiles() {
  var files = _fs2.default.readdirSync(_paths.appDir).reduce(function (filesObject, file) {
    var content = _fs2.default.readFileSync(_path2.default.join(_paths.appDir, file), 'utf8');

    filesObject[file] = {
      content: content.trim().length || content.trim() === '\n' ? content : 'EMPTY_CONTENT'
    };

    return filesObject;
  }, {});

  files.syncInfo = {
    content: (0, _jsonStableStringify2.default)({
      version: _version2.default,
      lastUpdated: new Date().toISOString()
    }) + '\n'
  };
  return files;
};

exports.default = function () {
  var repoType = (0, _config.getConfig)('repository.type');

  if (!repoType) throw new Error('Specify the repository type using \'icbm config set <key> <value>\'');

  switch (repoType) {
    case 'gist':
      var files = getUploadingFiles();

      return gist.get((0, _config.getConfig)('repository.gist')).then(function (res) {
        var deleted = Object.keys(res.data.files).filter(function (f) {
          return !(0, _lodash.has)(files, f);
        });
        deleted.forEach(function (f) {
          return files[f] = null;
        });
        return gist.edit(files).then(function () {
          return Promise.resolve({
            uploaded: _lodash.without.apply(undefined, [Object.keys(files)].concat(_toConsumableArray(deleted))),
            deleted: deleted
          });
        });
      });
    default:
      return Promise.reject({
        message: repoType + ' is not supported yet.',
        id: 'Invalid repository type'
      });
  }
};